%{
/* config.lex -- Tokenizer for the config file parser.
 * Dean Collins
 * created: Mon Aug 24 23:33:07 PDT 1992
 */

/*
 * Copyright (c) 1992 Dean Collins.
 * Copyright (c) 1992 University of Idaho, Moscow, Idaho.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation free of charge for any purpose is hereby granted without
 * fee, provided that the above copyright notices appear in all copies and
 * that both those copyright notices and this permission notice appear in
 * supporting documentation, and that neither the name of the University of
 * Idaho nor the name of Dean Collins be used in advertising or publicity
 * pertaining to distribution of the software without specific, written
 * prior permission from both parties.  Neither The University of Idaho
 * nor Dean Collins make any representations about the suitability of
 * this software for any purpose.  It is provided "as is" without express
 * or implied warranty.
 * 
 * THE UNIVERSITY OF IDAHO AND DEAN COLLINS DISCLAIM ALL WARRANTIES WITH
 * REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL THE UNIVERSITY OF IDAHO
 * OR DEAN COLLINS BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 */
%}

/* Defines go here */
slash   "/"
uname	[a-zA-Z0-9]+
pname	[^ #\t\n/]+
path	{pname}*({slash}+{pname})+
eol	"\n"
ws	[ \t]+
comment "#"[^\n]*


%{
/* Includes here */
#include <stdlib.h>
#include <string.h>
#include "y.tab.h"
#include "zdbm.h"
#include "cloud.h"

/* Things that make you go hmmm....*/
#ifdef DEBUG
#undef stderr
#define stderr stdout
int debug=1;
#else
int debug=0;
#endif
/* hmmm....*/

/* Function prototypes */
/* Make sure it's not already defined as a macro */
#ifdef yywrap
#undef yywrap
#endif /* yywrap */
int yywrap() ;

/* yylineno global variable doesn't appear to be define under Linux's
 * lex util.
 */
#ifdef linux
#define yylineno -1
#endif

/* Define vars. here. */
char ParserPath[DBPATHLEN] ;
char ParserName[MAXNAMELEN] ;
int ParserDays ;
extern int parse_error ;

%}
       /*--------------- Start of Lex expressions ------------------*/
%%


{ws}{eol}	{ if (debug) printf("[EOL]") ; return(EOL) ; }

{eol}		{ if (debug) printf("[EOL]") ; return(EOL) ; }

{ws}		{ if (debug) printf("[WS]") ; return(WS) ; }

{comment}{eol}	{ if (debug) printf("[EOL]") ; return(EOL) ; /*Turns comments into EOLs*/}

{comment}$	{ if (debug) printf("[EOL]") ; return(EOL) ; /*Turns comments into EOLs*/}

[0-9]+		{ if (debug) printf("[INT(%s)]", yytext) ; 
		  ParserDays = atoi((char *)yytext) ;
		  return(INT) ; 
		}

{uname}		{ if (debug) printf("[NAME(%s)]", yytext) ; 
                  strcpy(ParserName, (char *)yytext) ;
                  return(NAME) ; 
		}

{path}		{ if (debug) printf("[PATH(%s)]", yytext) ; 
                  strcpy(ParserPath, (char *)yytext) ;
                  return(PATH) ; 
		}

%%
/* Function defs here. */


int yywrap() { return(1) ; }

yyerror(s)
char *s;
{   printf("\n");
    fprintf(stderr, "*** config file parser error\n"
		    "*** %s, line=%d, text = '%s'\n", s, yylineno, yytext);
    parse_error++ ;
}

