/* Author:  Luke Sheneman.  */

/*
 * Copyright (c) 1992 University of Idaho, Moscow, Idaho.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation free of charge for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the University of Idaho
 * not be used in advertising or publicity pertaining to distribution of
 * the software without specific, written prior permission.  The University
 * of Idaho makes no representations about the suitability of this software
 * for any purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE UNIVERSITY OF IDAHO DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE UNIVERSITY OF IDAHO BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */


/*Folowing define modified 3/23/93, Dean Collins */
#ifndef SYSOPGID
#define SYSOPGID 25 
#endif

#define True 1
#define False 0
#define MAXNAME 128

#define MAXBRANCHES 512

#define MAXSUMMARIES 512	/* This is not a limit, but a throwback to 
					an old debugging function in 
					tester.c 
				*/

#define ABS_TYPE 0
#define INC_TYPE 1
#define PREV_TYPE 2

#define ROOTSTR "ROOT"

#define PTS_UMASK 022

#define TEMPLATE "/tmp/xptstmpXXXXXX"

/*Folowing defined modified 3/23/93, Dean Collins */
#ifndef MAILPROG
#define MAILPROG	"/usr/ucb/Mail"
#endif
/* The next two may need to be changed if your mailer doesn't use -s
 * to specify the subject.  DC
 */
#define MAILER 		MAILPROG " -s \"PTS Reopen Request\""
#define SOLVED_MAILER 	MAILPROG " -s \"PTS Solved Notice\""
#ifndef PRINTER
#define PRINTER		"/bin/prn"
#endif
#ifndef PRINTER_OPTS
#define PRINTER_OPTS 	"-HProblem_Description"
#endif

struct Full_path
{
	char path[DBPATHLEN];
	struct Full_path *next;
};

typedef struct problemtree
{
	struct problemtree *subhead;
	struct problemtree *next;
	char fullpath[DBPATHLEN];
	char path[DBPATHLEN];
} ProblemTree;

typedef struct sum
{
	char summary[MAXSUMLEN];
	char prid[MAXPRID];
	char path[DBPATHLEN];
	int status;
	struct sum *next;
} Summary;


typedef struct 
{
	int sysopflag;
	char *username;
	char *realname;
} UserInfo;

typedef struct mailinfo
{
        char username[MAXNAME];
        int days;
        struct mailinfo *next;
} MailInfo;



/*********************** Prototypes for setup.c *****************************/

int Setup(UserInfo *, ProblemTree **);
void BuildTree(ProblemTree **, struct Full_path *);
void BreakUpPath(char *,char *parts[MAXBRANCHES]);
void BreakUpPath2(char *,char parts[MAXBRANCHES][DBPATHLEN]);
void BuildTreePath(ProblemTree **, 
	char parts[MAXBRANCHES][DBPATHLEN], int);
ProblemTree *FindBranch(ProblemTree *, char *);
void AddNode(struct Full_path *, char *);
void TraverseLeaves(ProblemTree *, struct Full_path *);
void TraverseTree(ProblemTree *, struct Full_path *);
void TraverseList(struct Full_path *);




/********************* Prototypes for problems.c ***************************/

int FormatProblemTypes(ProblemTree *, char *, int, int, 
	char *types[]);
problem_record *ReadProblem(ProblemTree *, char *, int, Summary *);
int LockProblem(char *, problem_record *);
int UnlockProblem(char *, problem_record *);
int SolveProblem(ProblemTree *, char *, problem_record *, char *, UserInfo *);
problem_record *ReopenProblem(ProblemTree *, char *, problem_record *, char *);
int ReopenRequest(char *, problem_record *, char *, UserInfo *);
int DenyReopen(ProblemTree *, char *, problem_record *, UserInfo *, char *);
problem_record *ReportAgain(char *path, UserInfo *, problem_record *);
int DeleteProblem(ProblemTree *, char *, problem_record *);
int MoveProblem(ProblemTree *, char *, problem_record *, char *);
int LinkProblem(ProblemTree *, char *, problem_record *, char *);
int PrintProblem(ProblemTree *, char *, problem_record *);
int WriteProblem(ProblemTree *, char *, UserInfo *, char *, char *, char *);
int AppendProblem(ProblemTree *, char *, problem_record *, UserInfo *, char *);
int IsSysop(UserInfo *);
void CopyProblemRecord(problem_record *,problem_record *);
char *FixTime(char *);
int IsReopened(char *,problem_record *);
void FreeProblemRecord(problem_record *);






/********************* Prototypes for summaries.c ***************************/

Summary *ReadUnsolvedSummaries(ProblemTree *, char ***);
Summary *ReadSummaries(ProblemTree *, char *, char ***);
Summary *FormatSummaries(Summary *, Summary *, char ***);
Summary *FormatUnsolvedSummaries(Summary *, char ***);
void printsummaries(Summary *);
int CompPrid(char *, char *);
Summary *AddSummary(Summary *, problem_record *, char *);
problem_record *ReadUnsolved(ProblemTree *,char *,Summary *,int);
int FreeSummaries(Summary *);
char *FixPath(char *);
Summary *SortSummaries(Summary *);
Summary *AddSortedSummary(Summary *,Summary *);
void DummyTraverseSummaries(Summary *);
void FreeList(char **);





/********************** Prototypes for subtree.c ****************************/

void FreeParts(char *parts[MAXBRANCHES]);
ProblemTree *SubTree(ProblemTree *, char *);


/********************** Prototypes for tester.c *****************************/

void PrintUserInfo(UserInfo *);
int P_code(int, struct Full_path **);
void TraverseList(struct Full_path *);
void PrintParts(char parts[MAXBRANCHES][DBPATHLEN]);
void PrintUnsolvedSummaries(char unsummaries[MAXSUMMARIES][MAXSUMLEN]);
void TraverseSummaries(Summary *);
void printsummary(Summary *);
