/*
 * defines.h -- #defines for X interface to the Problem Tracking System
 * (PTS) database.
 * Dean Collins -- Mon Feb  8 21:27:40 PST 1993
 */

/*
 * Copyright (c) 1992,1993 Dean Collins.
 * Copyright (c) 1992 University of Idaho, Moscow, Idaho.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation free of charge for any purpose is hereby granted without
 * fee, provided that the above copyright notices appear in all copies and
 * that both those copyright notices and this permission notice appear in
 * supporting documentation, and that neither the name of the University of
 * Idaho nor the name of Dean Collins be used in advertising or publicity
 * pertaining to distribution of the software without specific, written
 * prior permission from both parties.  Neither The University of Idaho
 * nor Dean Collins make any representations about the suitability of
 * this software for any purpose.  It is provided "as is" without express
 * or implied warranty.
 * 
 * THE UNIVERSITY OF IDAHO AND DEAN COLLINS DISCLAIM ALL WARRANTIES WITH
 * REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL THE UNIVERSITY OF IDAHO
 * OR DEAN COLLINS BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 */


/* IMPORTANT NOTE:
 *   Most of the defines here can be over-ridden at compile time on
 *   the command-line.  This is the preferred way to change the
 *   defaults, rather than modifying this file.
 */


/* This string is displayed in two cases:
 *
 * 	1.  If xpts is installed set-uid root, and
 *
 *	2.  If xpts is executed by root.
 *
 * You do NOT want to run xpts as root.  It is a security risk.
 * If you disable this feature of xpts and manage to run xpts as root
 * you are foolish.  DO NOT RUN XPTS AS ROOT.  
 * Zombie software and all parties involved disclaim any and all damages
 * that result as a direct or indirect consequence of running xpts as root.
 * See the full disclaimer above.
 * 
 * If you do try to run xpts as root (DONT!) this string will be displayed
 * and xpts will immediatly exit.
 * 
 * If you have a problem with this contact the authors via e-mail.
 *
 * AGAIN, DO NOT DISABLE THIS.
 */
#define NO_ROOTUSERS \
"\n\nXpts cannot be executed by root, nor can it be installed setuid root.\n"\
"Sorry.\n\n\n"


/* Do you want xpts to keep a log of who uses it? Comment this define
 * out if you don't.  No attempt is made to trim the logfile.
 * This was used primarily in the testing phase.  You probably
 * won't need it.
 */
/*#define USERLOG */

#ifdef USERLOG
#ifndef LOGFILE
#define LOGFILE "/usr/local/lib/PTS/pts.log"
#endif /* LOGFILE */
#endif /* USERLOG */


/* The full path for the message-of-the-day file, in quotes. */
#ifndef MOTDFILE
#define MOTDFILE "/etc/motd"
#endif


/* These are strings used by the problem viewer to seperate the
 * various parts of the problem log.
 */
#ifndef PROBLEM_STR
#define PROBLEM_STR "PROBLEM DESCRIPTION:\n"
#endif

#ifndef REOPEN_REQ_STR
#define REOPEN_REQ_STR "REOPEN REQUEST REASON:\n"
#endif

#ifndef REOPEN_REQ_STR2
#define REOPEN_REQ_STR2 "\n\nORIGINAL PROBLEM LOG: \n" \
                        "---------------------------------------------------\n"
#endif

#ifndef LOG_ENTRY_STR
#define LOG_ENTRY_STR "\n\nLOG ENTRY:\t"
#endif

#ifndef SOLVED_ENTRY_STR
#define SOLVED_ENTRY_STR "\n\nPROBLEM SOLUTION:\t"
#endif

#ifndef REOPENED_ENTRY_STR
#define REOPENED_ENTRY_STR "\n\nPROBLEM REOPENED:\t"
#endif



/* This should be edited so that it informs the user that they
 * can delete problems that they themselves are editing.  (A glitch
 * in the current zdbm.)  
 */
#ifndef DELETE_WARNING
#define DELETE_WARNING "\n\
WARNING:  Once deleted, this problem will be permanently removed \n\
from the database.  If you are not sure you want to do this, \n\
Press the [Cancel] button below.  Otherwise, to delete the problem \n\
Press the [Continue] button below. \n"
#endif


#ifndef EMPTY_LOG_WARNING
#define EMPTY_LOG_WARNING "\n\
WARNING:  You have supplied an empty problem log.  Are you sure\n\
you want to make an empty log entry?  If not, press the [Cancel]\n\
button below.  Otherwise, to add the empty log entry to the\n\
problem log, press the [Continue] button below. \n"
#endif


#ifndef EMPTY_REQ_WARNING
#define EMPTY_REQ_WARNING "\n\
WARNING:  You have supplied an empty reopen request.  Are you sure\n\
you want to make an empty request?  If not, press the [Cancel]\n\
button below.  Otherwise, to add the empty reopen request to the\n\
database, press the [Continue] button below. \n"
#endif


#ifndef EMPTY_DESC_WARNING
#define EMPTY_DESC_WARNING "\n\
WARNING:  You have supplied an empty problem summary.  Are you sure\n\
you want to make an empty summary entry?  If not, press the [Cancel]\n\
button below.  Otherwise, to add problem with the empty problem\n\
summary to the problem log, press the [Continue] button below.\n"
#endif


#ifndef EMPTY_NODE_WARNING
#define EMPTY_NODE_WARNING "\n\
WARNING:  You have supplied an empty problem node value.  Are you\n\
sure you want to give an empty node entry?  If not, press the [Cancel]\n\
button below.  Otherwise, to add problem with the empty node\n\
value to the problem log, press the [Continue] button below.\n"
#endif


#ifndef REOPEN_NOTICE
#define REOPEN_NOTICE "\n\
NOTICE:  You have made a request to reopen this problem.\n\
Appropriate support personnel will be notified of your request.\n "
#endif

#ifndef DENY_REOP_NOTICE
#define DENY_REOP_NOTICE "\n\
NOTICE:  You have denied a request to reopen this problem.\n\
The problem reporter will be notified by E-mail of your denial.\n "
#endif


/* PRINTER and PRINTER_OPTS are defined in cloud.h.  This is a somewhat
 * strange definition of PRINTING_NOTICE.  It takes advantage of C's
 * merging of adjacent strings.
 */
#ifndef PRINTING_NOTICE
#define PRINTING_NOTICE "\n\
NOTICE:  This problem log has been queued to the printer.\n\
The following command was used to print the problem log:\n\n"\
PRINTER " " PRINTER_OPTS
#endif

#ifndef SOLVED_NOTICE
#define SOLVED_NOTICE "\n\
NOTICE:  The reporters of this problem have been notified via E-mail\n\
of your problem solution.\n"
#endif

#ifdef DEBUG
#define DEBUG_NOTICE "\n\
NOTICE:  This software is in the beta testing phase.  Any data you\n\
enter may be deleted or mangled by the software development team.\n\
If you are a system user who has discovered this program on your own,\n\
please quit Xpts immediatly.  There will be an announcement made when\n\
Xpts becomes generally available.  If you are officially participating\n\
in the beta testing of Xpts or are otherwise authorized by the\n\
software development team to use this software, please excuse this\n\
annoying notice.\n"
#else
#define DEBUG_NOTICE ""
#endif /*DEBUG*/

#ifndef STARTUP_NOTICE
#define STARTUP_NOTICE "\n"\
VERSION "\n" COPYRIGHT "\n\n" DEBUG_NOTICE
#endif /*STARTUP_NOTICE*/

#ifndef EMPTY_UNSOLVED
#define EMPTY_UNSOLVED "No Unsolved Problems and no Reopen Requests found"
#endif

#ifndef EMPTY_SUMMARIES
#define EMPTY_SUMMARIES "No problems found"
#endif


      /* Pretty generic definitions, used in various places.
       */

#ifndef TRUE
#define TRUE	1
#endif
#ifndef FALSE
#define FALSE	0
#endif


#define QUERY (-1) 		/* Flag used to check if a problem is locked */



/* NAMELEN is the maximum number of characters in a reporter record's
 * account field that will be displayed in the problem viewer. 
 * The account name may be longer, but extra characters will be ignored.
 */
#ifndef NAMELEN
#define NAMELEN 20
#endif

/* NODELEN is the maximum number of characters in a reporter record's
 * node field that will be displayed in the problem viewer. 
 * The account name may be longer, but extra characters will be ignored.
 */
#ifndef NODELEN
#define NODELEN 20
#endif


/* MAXTEXT is the maximum length of a problem record that can be displayed
 * in the problem viewer.  This will be eliminated in favor of dynamic
 * memory allocation (soon).
 */
#ifndef MAXTEXT
#define MAXTEXT 99999
#endif

/* MAXERROR is the maximum length of an error message.
 */
#ifndef MAXERROR
#define MAXERROR 1000
#endif

/* ReadProblem() takes one of two flags:
 *	UNSOLVEDPROB	-- Read the i'th unsolved problem.
 *	ANYPROB		-- Read the i'th problem, solved or not.
 *
 * "i" is the index into the list of summaries being shown.  
 * The first (top) element in the list is 0.
 */
#define UNSOLVEDPROB	69
#define ANYPROB		70


#define PTS_INFO_STR \
"______________________________________________________________________	\n" \
"      Problem Tracking System						\n" \
"      Xpts Version 1.03						\n" \
"      Zombie Software							\n" \
"									\n" \
"      PTS is a database tool which is used to improve communication	\n" \
" between system users and support personnel about problems encountered	\n" \
"with the system.							\n" \
"     Xpts is an X Window System application used to access the		\n" \
"PTS database in an X environment.					\n" \
"									\n" \
"Copyright (c) 1992,1993 Dean Collins, Chris Hyde, Luke Sheneman,	\n" \
"                        Brian Goff, and John Hunter.			\n" \
"Copyright (c) 1992 University of Idaho, Moscow, Idaho.			\n" \
"Copyright (c) 1985, 1986, 1987, 1988, 1989 Massachusetts		\n" \
"      Institute of Technology, Cambridge, Massachusetts, and		\n" \
"      Digital Equipment Corporation, Maynard, Massachusetts.		\n" \
"The X Window System is a trademark of MIT.				\n" \
"______________________________________________________________________	\n" \
"									\n" \
"The Problem Tracking System was designed by Zombie Software		\n" \
"at the University of Idaho.						\n" \
"									\n" \
"Zombie Software is:							\n" \
"									\n" \
"      Project Leader:							\n" \
"									\n" \
"            Dean Collins, Assistant System Manager			\n" \
"                  X11 user-interface design and implementation,	\n" \
"		   Configuration file parser design and implementation.	\n" \
"                  (Internet: dean@cs.uidaho.edu, dean@uidaho.edu)	\n" \
"									\n" \
"      Design Team Members:						\n" \
"									\n" \
"            Chris Hyde							\n" \
"                  zdbm database design/implementation.			\n" \
"		   (Internet: chyde@delphi.com)				\n" \
"									\n" \
"            Brian Goff							\n" \
"                  cron sub-system design and implementation.		\n" \
"		   (Internet: brian@snake.cs.uidaho.edu)		\n" \
"									\n" \
"            John Hunter						\n" \
"		   Software Requirements Document and Software		\n" \
"		   Design Document coordination.			\n" \
"		   (Internet: hunter874@snake.cs.uidaho.edu)		\n" \
"									\n" \
"            Luke Sheneman						\n" \
"                  High-level database interface.			\n" \
"		   Configuration file parser design and implementation.	\n" \
"		   (Internet: sheneman@snake.cs.uidaho.edu)		\n" \
"									\n" \
"Special thanks to: 							\n" \
"	Prof. John Dickinson, Chair, Computer Science Department,	\n" \
"		(for the idea);						\n" \
"	Mike Simon, System Manager, (for the patience and guidance);	\n" \
"	Prof. Karen Van Houten, (for the suggestions);			\n" \
"	Brent Heikkilia (for help with the Requirements Analysis);	\n" \
"	and Dayne Steed (for early Requirements and Design work).	\n" \
"									\n" \
"Please direct E-mail to one of the following Internet addresses:	\n" \
"	dean@cs.uidaho.edu (preferred)					\n" \
"	dean@uidaho.edu							\n" \
"	zombie@snake.cs.uidaho.edu					\n" \
"	support@snake.cs.uidaho.edu					\n" \
"	simon@cs.uidaho.edu						\n" \
"									\n" \
"THANK YOU FOR USING XPTS!						\n" \
" 									\n" \
"______________________________________________________________________	\n" \
"DISCLAIMER:								\n" \
"      The University of Idaho, M.I.T. and Zombie Software make no	\n" \
"representations about the suitability of this software for any purpose.\n" \
"It is provided 'as is' without express or implied warranty.		\n" \
"______________________________________________________________________	\n" 


/* end of defines.h */
