/* 
 * zdbm_io.c --
 * Chris Hyde
 * created: 8/92
 */

/*
 * Copyright (c) 1992 Christopher G. Hyde
 * Copyright (c) 1992 University of Idaho, Moscow, Idaho.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation free of charge for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the University of Idaho
 * not be used in advertising or publicity pertaining to distribution of
 * the software without specific, written prior permission.  The University
 * of Idaho makes no representations about the suitability of this software
 * for any purpose.  It is provided "as is" without express or implied warranty.
 * 
 * THE UNIVERSITY OF IDAHO DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE UNIVERSITY OF IDAHO BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */


#ifndef SYSV

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <time.h>
#include <errno.h>
#include <limits.h>
#ifdef linux
#include <unistd.h>
#endif /* linux */
/* end not system 5 */

#else /* it is system 5 */

#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#ifdef AIXV3
#include <sys/time.h>
#else
#include <time.h>
#endif /* AIXV3 */
#include <errno.h>

#endif /* system 5 */

#include "zdbm.h"

#ifdef DEBUG
 int open_closed=0,
     locked_unlocked=0;
#endif


int exists(char *root, char *path, char *fname, char *extension)
{
 static char tpath[COMPLETEPATHLEN];

#ifdef DEBUG
 printf("\nIn exists()\n");
 printf("\tPath - %s%s\n", root, path);
 printf("\tFile name to check - %s\n", fname);
 printf("\tFile extension - %s\n", extension);
#endif

 if ((fname[0]=='\0') && (extension[0]=='\0'))
 	/* no file name or extension so don't add a '/' */
    sprintf(tpath, "%s%s", root, path);
 else
	/* we're going to need the '/' */
    sprintf(tpath, "%s%s/%s%s", root, path, fname, extension);
 if (access(tpath, F_OK)==ERROR) 
 {
#ifdef DEBUG
 printf("exists() - the file %s does NOT exist\n", tpath);
#endif

    db_errorno=errno;
    return(FALSE);
 }
 else
 {
#ifdef DEBUG
 printf("exists() - the file %s does exist\n", tpath);
#endif

    return(TRUE);
 }
} /* end exists */


int append(char *path, char *from1, char *from2, char *to1, char *to2)
{
 FILE *temp1, *temp2;
 char file1[COMPLETEPATHLEN],
      file2[COMPLETEPATHLEN],
      buffer[STD_BLK];
 long bottom;

 sprintf(file1, "%s/%s", path, from1);
 sprintf(file2, "%s/%s", path, to1);

#ifdef DEBUG
 printf("\nIn append()\n");
 printf("\tAppending %s%s\nTo %s%s\n", file1, from2, file2, to2);
#endif

 if ((temp1=open_file(file1, from2, "r", REP))==NULL)
    return(FAIL); /* could not open "from" file */
 if ((temp2=open_file(file2, to2, "a", REP))==NULL)
 {      /* could not open "to" file */
    close_file(temp1);
    return(FAIL);
 }

bottom=ftell(temp2);

 if (lock_file(temp2)==FAIL)
 {      /* the lock failed */
    close_file(temp1);
    close_file(temp2);
    return(FAIL);
 }

#ifdef DEBUG
 printf("append() - performing append - ");
#endif

 while (fread(buffer, STD_BLK, 1, temp1)==1)
 {      /* read from the "from" file until the end */

#ifdef DEBUG
 printf("?");
#endif

    if (fwrite(buffer, STD_BLK, 1, temp2)<1)
       break; /* there was an error writting to the "to" file */
 }

#ifdef DEBUG
 printf("\n");
#endif

 unlock_file(temp2);

 if (ferror(temp1) || ferror(temp2))
 {      /* there was an error either reading or writting */
    db_errorno=errno;
    close_file(temp1);
    close_file(temp2);
    trunc_file(file2, to2, bottom);
    return(FAIL);
 }
 else
 {
    close_file(temp1);
    close_file(temp2);
    return(SUCESS);
 }
} /* end append() */


FILE *open_file(char *leaf, char *extension, char *perm, int index)
{
   char path[COMPLETEPATHLEN],
        *temp;
   int fd, mode;

   if (index==INDEX)  /* want to open an index file */
   {
      if ((temp=rindex(leaf, '/'))==NULL) /* find the name of the leaf */
         temp=leaf; /* name of the index is the name fo the leaf */
      else
         temp++; /* want the char after the '/' */
      sprintf(path, "%s%s/%s%s", PTSROOT, leaf, temp, extension);
   }
   else /* not opening an index file */
      sprintf(path, "%s%s%s", PTSROOT, leaf, extension);

#ifdef DEBUG
 printf("\nIn open_file()\n");
 printf("\tFile To Open %s\n", path);
 printf("\tAcess method = %s\n", perm);
#endif

   if (perm[1]=='+') /* open for both reading and writing */
      mode=O_RDWR;
   else
   {
      if (perm[0]=='a') /* open for appending */
         mode=O_APPEND | O_WRONLY;
      else
         if (perm[0]=='r') /* open for read only */
            mode=O_RDONLY;
         else
            if (perm[0]=='w') /* open for writting only */
               mode=O_WRONLY;
   } /* end else */
   if ((fd=open(path, mode, FILE_PERMS))<0)
   {    /* error opening file */

#ifdef DEBUG
 printf("open_file() - could not open %s\n", path);
 printf("\terror number %d\n", errno);
#endif

      db_errorno=errno;
      return(NULL);
   }
   else
   {
#ifdef DEBUG
 printf("open_file() - opened %s\n", path);
 printf("\tOpens - Closes = %d\n", ++open_closed);
#endif
      return(fdopen(fd, perm));
   } /* end else */
} /* end open_file */


int close_file(FILE *db_pointer)
{
int status;

#ifdef DEBUG
 printf("In close_file()\n\tOpens - Closes = %d\n", --open_closed);
#endif

 status=(fclose(db_pointer));

#ifdef DEBUG
printf("\tstatus=%d\n",status) ;
#endif

 return(status) ;
} /* end close_file */


long file_size(FILE *fp, long record_size)
{
  long size;
  struct stat statbuf; /* space to hold info about the file */

#ifdef DEBUG
  printf("\nIn file_size()\n");
  printf("\trecord_size = %d\n", record_size);
#endif

  if (fstat(fileno(fp), &statbuf)==ERROR) /* get the file status */
  {	/* error getting file status */
     db_errorno=errno;
     return(ERROR);
  }
  size=(long)statbuf.st_size; /* size of file in bytes */
  return(size/record_size);
} /* end file_size */


int lock_file(FILE *fp)
{
  int fd; /* file descriptor */

#ifdef DEBUG
  printf("\nIn lock_file()\n");
#endif

  fd=fileno(fp);
#ifdef DEBUG
  printf("\tfd = %d, fp = %d, ferror(fp) = %d\n",fd, fp, ferror(fp)) ;
#endif

#ifndef SYSV /* bsd specific file locking */

  if (flock(fd, LOCK_EX | LOCK_NB)<0)
  {  /* lock failed */
#ifdef DEBUG
     printf("\terrno = %d\n",errno) ;
#endif
     if (errno==EWOULDBLOCK)
        db_errorno=ETXTBSY; /* used for consistancy */
     else
        db_errorno=errno;
     return(FAIL);
  }
  else /* lock succeeded */
  {
#ifdef DEBUG
 printf("\tLocked - Unlocked = %d\n", ++locked_unlocked);
#endif
     return(SUCESS);
  } /* end else */
 /* end of BSD specific file locking */

#else /* it is system 5 specific file locking */

  if (lockf(fd, F_TLOCK, 0)==ERROR)
  {  /* lock failed */
     if (errno==EACCES) /* file is already locked */
        db_errorno=ETXTBSY; /* used for consistancy */
     else
        db_errorno=errno;
     return(FAIL);
  }
  else /* lock succeeded */
  {
#ifdef DEBUG
 printf("\tLocked - Unlocked = %d\n", ++locked_unlocked);
#endif
     return(SUCESS);
  } /* end else */
#endif /* end of system 5 specific file locking */
} /* end lock_file */


int unlock_file(FILE *fp)
{
  int fd; /* file descriptor */

#ifdef DEBUG
  printf("\nIn unlock_file()\n");
#endif

  fd=fileno(fp);

#ifndef SYSV /* begin of BSD specific file unlocking */
  if (flock(fd, LOCK_UN)<0)
  {  /* unlocking failed */
     db_errorno=errno;
     return(FAIL);
  }
  else /* unlocking suceeded */
  {
#ifdef DEBUG
 printf("\tLocks - Unlocks = %d\n", --locked_unlocked);
#endif

     return(SUCESS);
  } /* end else */
 /* end of system 5 specfic file unlocking */

#else /* begin of system 5 specific file unlocking */
  if (lockf(fd,F_ULOCK, 0)==ERROR)
  {  /* unlocking failed */
     db_errorno=errno;
     return(FAIL);
  }
  else /* unlocking suceeded */
  {
#ifdef DEBUG
 printf("\tLocks - Unlocks = %d\n", --locked_unlocked);
#endif

     return(SUCESS);
  } /* end else */
#endif /* end of system 5 specific file locking */
} /* end unlock_file */


int make_branch(char *db_path)
{
  char path[COMPLETEPATHLEN];
  char *cp, 
       tpath[DBPATHLEN]; /* since strtok plays with the string passed to it
			  * we need to place the db_path into another string
			  * so the original won't be affected */
  FILE *dummy;

  strcpy(path, PTSROOT); /* copy the database root into the path */
  nl_strncpy(tpath, db_path, DBPATHLEN);

#ifdef DEBUG
  printf("\nIn make_branch()\n");
  printf("\tdb_path = %s\n", db_path);
#endif

  cp=strtok(tpath, "/"); /* get the first part of the path */
  strcat(path, cp); /* place the first part of the new branch in path */

  while(cp!=NULL) /* while there are more partial paths */
  {
#ifdef DEBUG
  printf("make_branch() - partial path = %s\n", path);
#endif
     if (exists("", path, "", "")==FAIL)
 /* if the partial path does not exist, then create it with the permissions
  * as defined by DIR_PERMS (in zdbm.h)
  */
        if (mkdir(path, DIR_PERMS)==ERROR)
        { /* error creating directory */
           db_errorno=errno;
           return(FAIL);
        }
     cp=strtok(NULL, "/"); /* get next portion of pathlist */
     if (cp!=NULL)
     {
        strcat(path, "/");
        strcat(path, cp);
     }
  } /* end while */

/* we made the directory path, now make the new index file */

  cp=rindex(path, '/'); /* get the name of the last directory in the list */
  if (cp==NULL) /* the path has only one directory */
     cp=db_path;
  else
     cp++; /* we want the char after the '/' */
/* create the index file, since create_file() leaves the created file open
 * we have to close it before exiting from the module.
 */
  if ((dummy=create_file(db_path, cp, ".index", FILE_PERMS))==NULL)
     /* error occured durring the creation */
     return(FAIL);
  else
  {
     close_file(dummy);
     return(SUCESS);
  }
} /* end make_branch */


FILE *create_file(char *path, char *fname, char *extension, int perm)
{
 char tpath[COMPLETEPATHLEN];
 int fd;

#ifdef DEBUG
  printf("\nIn create_file()\n\tpath = %s%s\n", PTSROOT, path);
  printf("\tfname = %s\n\textension = %s\n\tperm = %d\n", fname,
		 extension, perm);
#endif

 sprintf(tpath, "%s%s/%s%s", PTSROOT, path, fname, extension);
 if ((fd=open(tpath, O_WRONLY | O_CREAT | O_EXCL, perm))<0)
 {  /* error creating file */
    db_errorno=errno;
    return(NULL);
 }
#ifdef DEBUG
 printf("create_file() - created %s\n", tpath);
 printf("\tOpened - Closed = %d\n", ++open_closed);
#endif

 return(fdopen(fd, "w"));
} /* end create_file */


int remove_file(char *path, char *fname, char *extension)
{
 char tpath[COMPLETEPATHLEN];

#ifdef DEBUG
 printf("\nIn remove_file()\n");
 printf("\tpath = %s%s\n\tfname = %s\n\textension = %s\n", PTSROOT, 
		path, fname, extension);
#endif

 sprintf(tpath, "%s%s/%s%s", PTSROOT, path, fname, extension);
 if (unlink(tpath)==ERROR)
 {
    db_errorno=errno;
    return(FAIL);
 }
 else
    return(SUCESS);
} /* end remove_file */


char *new_prid(long size)
{
 struct timeval clock;
 struct timezone tz;
 static char prid[MAXPRID];

#ifdef DEBUG
 printf("\nIn new_prid()\n");
 printf("\tsize = %d.\n", size);
#endif

 if (gettimeofday(&clock, &tz)==ERROR)
 {
    db_errorno=errno;
    return(NULL);
 }
 if (strftime(prid, MAXPRID, "%j%Y", localtime(&(clock.tv_sec)))==FAIL)
 {
    db_errorno=STRINGERROR;
    return(NULL);
 }
 sprintf(prid, "%s%05d", prid, size);
 return(prid);
} /* end new_prid */


int trunc_file(char *path, char *extension, long flen)
{
 char tpath[COMPLETEPATHLEN];

#ifdef DEBUG
 printf("\nIn trunc_file()\n");
 printf("\tPath = %s\n\tExtension = %s\n\tAmount = %d\n", path,
                extension, flen);
#endif

 sprintf(tpath, "%s%s", path, extension);
 if (truncate(tpath, flen)==ERROR)
 {  /* error truncating file */
    db_errorno=errno;
    return(FAIL);
 }
 else
    return(SUCESS);
} /* end trunc_file */


int rename_file(char *from_path, char *from1, char *from2, 
		char *to_path, char *to1, char *to2)
{
 char tpath1[COMPLETEPATHLEN],
      tpath2[COMPLETEPATHLEN];

#ifdef DEBUG
 printf("\nIn rename_file().\n");
 printf("\tRenaming %s%s%s\n to \n\t%s%s%s\n", from_path, from1, from2, 
 to_path, to1, to2);
#endif

 sprintf(tpath1, "%s%s/%s%s", PTSROOT, from_path, from1, from2);
 sprintf(tpath2, "%s%s/%s%s", PTSROOT, to_path, to1, to2);

 if (rename(tpath1, tpath2)==ERROR)
 {  /* error renameing file */
    db_errorno=errno;
    return(FAIL);
 }
 return(SUCESS);
} /* end rename_file */


int copy_file(char *from1, char *from2, char *from3,
              char *to1, char *to2, char *to3)
{
 char from[FNAMELEN],
      buffer[STD_BLK];
 FILE *infile,
      *outfile;
 int in;

#ifdef DEBUG
 printf("\nIn copy_file()\n");
 printf("\tFrom1 - %s\n\tFrom2 - %s\n\tFrom3 - %s\n", from1, from2,
		 from3);
 printf("\tTo1 - %s\n\tTo2 - %s\n\tTo3 - %s\n", to1, to2, to3);
#endif

 sprintf(from, "/%s%s", from2, from3);
 if ((infile=open_file(from1, from, "r", REP))==NULL)
    return(FAIL);
 if ((outfile=create_file(to1, to2, to3, FILE_PERMS))==NULL)
 {
    close_file(infile);
    return(FAIL);
 }

#ifdef DEBUG
 printf("Copying File");
#endif

 while ((in=fread(buffer, sizeof(char), STD_BLK, infile))!=NULL)
 {

#ifdef DEBUG
 putc('!', stderr);
#endif

    if (fwrite(buffer, sizeof(char), in, outfile)<in)
       break;
 }

#ifdef DEBUG
 putc('\n', stderr);
#endif

 if (ferror(infile) || ferror(outfile))
 {      /* error reading or writting */
    db_errorno=errno;
    close_file(infile);
    close_file(outfile);
    remove_file(to1, to2, to3);
    return(FAIL);
 }
 close_file(infile);
 close_file(outfile);
 return(SUCESS);
} /* end copy_file */


int write_log(char *path, char *prid, char *new_log)
{
 char tname[FNAMELEN];
 FILE *temp;

#ifdef DEBUG
 printf("\nIn write_log()\n");
 printf("\tLog path - %s%s/%s.log\n", PTSROOT, path, prid);
#endif

 nl_strncpy(tname, TEMPORARY, FNAMELEN);
 if ((temp=create_file(path, (char *)mktemp(tname), "", FILE_PERMS))
			==NULL)
    return(FAIL); /* the creation failed */

#ifdef DEBUG
 printf("write_log() - created new log, now writting it out\n");
#endif

/* we don't want to write the terminating null out to the file.  We will
 * add it back on we we read the log in
 */
 if (fwrite(new_log, sizeof(char), strlen(new_log), temp)<strlen(new_log))
 {      /* the write failed */
    db_errorno=errno;
    close_file(temp);
    return(FAIL);
 }
 else   /* the write suceeded */
 {
#ifdef DEBUG
 printf("write_log() - wrote file for new log\n");
#endif

    close_file(temp);
#ifdef DEBUG
 printf("write_log() - renaming temp file to log file\n");
#endif

    if (rename_file(path, tname, "", path, prid, ".log")==FAIL)
    {   /* couldn't rename the new problem log */
       remove_file(path, tname, "");
       return(FAIL);
    }
    else /* rename suceeded */
       return(SUCESS);
 } /* end else write suceeded */
} /* end write_log() */


char *zdbm_strerror(int zerror)
{
 int index;
 static char *error_messages[]=
	{
	 "BADTYPE - a bad parameter was passed to a database function.\n",
	 "NOTFOUND - the problem record asked for was not found in the leaf given.\n",
	 "DOUBLEREP - a reporter has tried to report a problem twice.\n",
	 "DELETED - the problem record asked for has been deleted.\n",
	 "ISSOLVED - the problem record given is solved.\n",
	 "ISUNSOLVED - the problem record given is unsloved.\n",
	 "NOTOWNER - the person requesting the end of an edit session is not the one who started it.\n",
	 "NOREP - the problem record is missing either the old reporters list, or the reporters list.\n",
	 "EXISTS - the problem requested has already been requested for reopening.\n",
	 "LINKERR - permission to delete a problem record that is an original link has not been given.\n",
	 "REMOVEERROR - an error occured while a record was being removed.\n",
	 "STRINGERROR - an error occured during a string operation.\n",
	 "ROLLEDOVER - there are no problem records of the desired type in the database leaf given.\n",
	 "SAMELEAF - a request was made to link or move a problem to a leaf in which it already resides.\n"
	}; /* end of error_messages */

#ifdef DEBUG
 printf("In zdbm_strerror()\n");
 printf("\tzdbm error number = %d\n", zerror);
#endif

 if (zerror<BEGIN_OF_ZDBM_ERROR_CODES) 
    return(strerror(zerror)); /* the error code is for a lib function */
 else
    if (zerror>END_ZDBM_ERROR_CODES) /* zerror is above the range for zdbm */
    {
       db_errorno=BADTYPE;
       return("Unknown ZDBM error code.\n");
    } /* end else zerror above range */
    else /* zerror is in the range of zdbm codes */
    {
       index=zerror-BEGIN_OF_ZDBM_ERROR_CODES;
       return(error_messages[index]);
    } /* end else  zerror is in range */
} /* end of zdbm_strerror() */

/* end of zdbm_io.c */
